@@ -10,6 +10,7 @@
 ## 2011 11 28 user dframe factors now covered
 ## 2015-10-08 'ts'
 ## 2017-12-18 all.levels argument
+## 2021-03-24 fix all.levels = FALSE bug
 ## Create (neutral) design data suitable for 'predict'
 ############################################################################################
 
@@ -116,9 +117,11 @@ secr.make.newdata <- function (object, all.levels = FALSE) {
         basevars <- findvars (basevars, timecov)
         basevars <- findvars (basevars, covariates(traps(capthist)))
         basevars <- findvars (basevars, covariates(mask))
-
+        
         ## revert to first level (original default)
-        if (length(v)>0) {
+        ## 2021-03-24 repaired in 4.3.4
+        for (v in names(basevars)) {
+            # if (length(v)>0) { 
             if (!all.levels & !(v %in% c('session', 'g', 'h2','h3'))) {
                 basevars[[v]] <- basevars[[v]][1] 
             }
